/*
 *  This file is part of uIRC. (https://git.redxen.eu/caskd/uIRC)
 *  Copyright (c) 2019-2021 Alex-David Denes
 *
 *  uIRC is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  uIRC is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with uIRC.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "memory.h"     // uirc_memory_*()
#include "tokenizers.h" // uirc_tokenizer_*()
#include "types.h"      // IRC_Message

#include <assert.h>             // assert()
#include <corelibs/stringext.h> // stringext_strmalloc()
#include <stdbool.h>            // bool
#include <stdlib.h>             // malloc()
#include <string.h>             // strchr()

static void free_ctx(IRC_User* u, char* s);

IRC_User*
uirc_tokenizer_user(const char* str)
{
	assert(str != NULL);

	IRC_User*   u  = malloc(sizeof(IRC_User));
	char* const ws = stringext_strmalloc(str, strlen(str));
	char*       tmp;

	if (u == NULL || ws == NULL) {
		free(u);
		free(ws);
		return NULL;
	}

	memset(u, 0, sizeof(IRC_User));

	if ((tmp = strchr(ws, '@')) != NULL) {
		*(tmp++) = '\0';
		if ((u->host = stringext_strmalloc(tmp, strlen(tmp))) == NULL) {
			free_ctx(u, ws);
			return NULL;
		}
	}
	if ((tmp = strchr(ws, '!')) != NULL) {
		*(tmp++) = '\0';
		if ((u->user = stringext_strmalloc(tmp, strlen(tmp))) == NULL) {
			free_ctx(u, ws);
			return NULL;
		}
	}
	if ((u->nick = stringext_strmalloc(ws, strlen(ws))) == NULL) {
		free_ctx(u, ws);
		return NULL;
	}

	free(ws);
	return u;
}

static void
free_ctx(IRC_User* u, char* s)
{
	uirc_struct_free(u, IRC_STRUCT_USER);
	free(u);
	free(s);
}
